@@ -22,7 +22,25 @@ $LOAD_PATH.unshift(mkmf_gnome2_dir.to_s)
 module_name = "atk"
 package_id = "atk"
 
-require 'mkmf-gnome2'
+begin
+  require 'mkmf-gnome2'
+rescue LoadError
+  require 'rubygems'
+  gem 'glib2'
+  require 'mkmf-gnome2'
+end
+
+ruby_header = 'ruby.h'
+have_func 'rb_errinfo', ruby_header
+
+["glib2"].each do |package|
+  directory = "#{package}#{version_suffix}"
+  build_dir = "#{directory}/tmp/#{RUBY_PLATFORM}/#{package}/#{RUBY_VERSION}"
+  add_depend_package(package, "#{directory}/ext/#{package}",
+                     top_dir.to_s,
+                     :top_build_dir => top_build_dir.to_s,
+                     :target_build_dir => build_dir)
+end
 
 setup_win32(module_name, base_dir)
 
@@ -42,17 +60,6 @@ have_func('atk_text_clip_type_get_type', atk_header)
 
 have_func('atk_text_free_ranges', atk_header)
 
-["glib2"].each do |package|
-  directory = "#{package}#{version_suffix}"
-  build_dir = "#{directory}/tmp/#{RUBY_PLATFORM}/#{package}/#{RUBY_VERSION}"
-  add_depend_package(package, "#{directory}/ext/#{package}",
-                     top_dir.to_s,
-                     :top_build_dir => top_build_dir.to_s,
-                     :target_build_dir => build_dir)
-end
-
-add_distcleanfile("rbatkinits.c")
-
 make_version_header("ATK", package_id, ".")
 
 create_pkg_config_file("Ruby/ATK", package_id)
