@@ -1,19 +1,30 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/************************************************
-
-  rbgtkstyle.c -
-
-  $Author: ggc $
-  $Date: 2007/07/13 16:07:32 $
-
-  Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
-  Copyright (C) 1998-2000 Yukihiro Matsumoto,
-                          Daisuke Kanda,
-                          Hiroshi Igarashi
-************************************************/
+/*
+ *  Copyright (C) 2011  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002,2003 Ruby-GNOME2 Project Team
+ *  Copyright (C) 1998-2000 Yukihiro Matsumoto,
+ *                          Daisuke Kanda,
+ *                          Hiroshi Igarashi
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA  02110-1301  USA
+ */
 
 #include "global.h"
 
+#define RG_TARGET_NAMESPACE cStyle
 #define _SELF(w) GTK_STYLE(RVAL2GOBJ(w))
 #define RVAL2REC(r) (NIL_P(r) ? NULL : (GdkRectangle*)RVAL2BOXED(r, GDK_TYPE_RECTANGLE))
 #define RVAL2ICONSOURCE(s) ((GtkIconSource*)RVAL2BOXED(s, GTK_TYPE_ICON_SOURCE))
@@ -24,46 +35,40 @@
 #define RVAL2ORI(s) (RVAL2GENUM(s, GTK_TYPE_ORIENTATION))
 
 static VALUE
-style_initialize(self)
-    VALUE self;
+rg_initialize(VALUE self)
 {
     G_INITIALIZE(self, gtk_style_new());
     return Qnil;
 }
 
 static VALUE
-style_attached(self)
-    VALUE self;
+rg_attached_p(VALUE self)
 {
     return CBOOL2RVAL(GTK_STYLE_ATTACHED(_SELF(self)));
 }
 
 static VALUE
-style_copy(self)
-    VALUE self;
+rg_copy(VALUE self)
 {
     return GOBJ2RVAL(gtk_style_copy(_SELF(self)));
 }
 
 static VALUE
-style_attach(self, win)
-    VALUE self, win;
+rg_attach(VALUE self, VALUE win)
 {
     return GOBJ2RVAL(gtk_style_attach(_SELF(self),
                                       GDK_WINDOW(RVAL2GOBJ(win))));
 }
 
 static VALUE
-style_detach(self)
-    VALUE self;
+rg_detach(VALUE self)
 {
     gtk_style_detach(_SELF(self));
     return self;
 }
 
 static VALUE
-style_set_background(self, win, state_type)
-    VALUE self, win, state_type;
+rg_set_background(VALUE self, VALUE win, VALUE state_type)
 {
     gtk_style_set_background(_SELF(self), GDK_WINDOW(RVAL2GOBJ(win)),
                              RVAL2STATE(state_type));
@@ -71,9 +76,7 @@ style_set_background(self, win, state_type)
 }
 
 static VALUE
-style_apply_default_background(self, gdkwindow, set_bg, state_type, area,
-                               x, y, width, height)
-    VALUE self, gdkwindow, set_bg, state_type, area, x, y, width, height;
+rg_apply_default_background(VALUE self, VALUE gdkwindow, VALUE set_bg, VALUE state_type, VALUE area, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_style_apply_default_background(_SELF(self), 
                                        GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
@@ -86,8 +89,7 @@ style_apply_default_background(self, gdkwindow, set_bg, state_type, area,
 
 #if GTK_CHECK_VERSION(2,10,0)
 static VALUE
-style_lookup_color(self, color_name)
-    VALUE self, color_name;
+rg_lookup_color(VALUE self, VALUE color_name)
 {
     GdkColor color;
     if (gtk_style_lookup_color(_SELF(self), RVAL2CSTR(color_name), &color)){
@@ -99,8 +101,7 @@ style_lookup_color(self, color_name)
 #endif
 
 static VALUE
-style_lookup_icon_set(self, stock_id)
-    VALUE self, stock_id;
+rg_lookup_icon_set(VALUE self, VALUE stock_id)
 {
     const gchar *id;
     GtkIconSet *ret;
@@ -109,14 +110,13 @@ style_lookup_icon_set(self, stock_id)
     } else {
         id = rb_id2name(SYM2ID(stock_id));
     }
-    
+
     ret = gtk_style_lookup_icon_set(_SELF(self), id);
     return BOXED2RVAL(ret, GTK_TYPE_ICON_SET);
 }
 
 static VALUE
-style_render_icon(self, source, direction, state, size, widget, detail)
-    VALUE self, source, direction, state, size, widget, detail;
+rg_render_icon(VALUE self, VALUE source, VALUE direction, VALUE state, VALUE size, VALUE widget, VALUE detail)
 {
     return GOBJ2RVAL(gtk_style_render_icon(_SELF(self), RVAL2ICONSOURCE(source),
                                            RVAL2GENUM(direction, GTK_TYPE_TEXT_DIRECTION), 
@@ -127,10 +127,7 @@ style_render_icon(self, source, direction, state, size, widget, detail)
 }
 
 static VALUE
-style_paint_arrow(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                  arrow_type, fill, x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    arrow_type, fill, x, y, width, height;
+rg_paint_arrow(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE arrow_type, VALUE fill, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_arrow(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -141,10 +138,7 @@ style_paint_arrow(self, gdkwindow, state_type, shadow_type, area, widget, detail
 }
 
 static VALUE
-style_paint_box(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_box(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_box(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -154,10 +148,7 @@ style_paint_box(self, gdkwindow, state_type, shadow_type, area, widget, detail,
 }
 
 static VALUE
-style_paint_box_gap(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                  x, y, width, height, gap_side, gap_x, gap_width)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height, gap_side, gap_x, gap_width;
+rg_paint_box_gap(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height, VALUE gap_side, VALUE gap_x, VALUE gap_width)
 {
     gtk_paint_box_gap(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                       RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -168,10 +159,7 @@ style_paint_box_gap(self, gdkwindow, state_type, shadow_type, area, widget, deta
 }
 
 static VALUE
-style_paint_check(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                  x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_check(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_check(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -181,10 +169,7 @@ style_paint_check(self, gdkwindow, state_type, shadow_type, area, widget, detail
 }
 
 static VALUE
-style_paint_diamond(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                  x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_diamond(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_diamond(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -194,10 +179,7 @@ style_paint_diamond(self, gdkwindow, state_type, shadow_type, area, widget, deta
 }
 
 static VALUE
-style_paint_extension(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                      x, y, width, height, gap_side)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height, gap_side;
+rg_paint_extension(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height, VALUE gap_side)
 {
     gtk_paint_extension(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                         RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -208,10 +190,7 @@ style_paint_extension(self, gdkwindow, state_type, shadow_type, area, widget, de
 }
 
 static VALUE
-style_paint_flat_box(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                     x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_flat_box(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_flat_box(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -221,10 +200,7 @@ style_paint_flat_box(self, gdkwindow, state_type, shadow_type, area, widget, det
 }
 
 static VALUE
-style_paint_focus(self, gdkwindow, state_type, area, widget, detail,
-                  x, y, width, height)
-    VALUE self, gdkwindow, state_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_focus(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_focus(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2REC(area),
@@ -234,10 +210,7 @@ style_paint_focus(self, gdkwindow, state_type, area, widget, detail,
 }
 
 static VALUE
-style_paint_handle(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                  x, y, width, height, orientation)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height, orientation;
+rg_paint_handle(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height, VALUE orientation)
 {
     gtk_paint_handle(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                      RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -248,8 +221,7 @@ style_paint_handle(self, gdkwindow, state_type, shadow_type, area, widget, detai
 }
 
 static VALUE
-style_paint_hline(self, gdkwindow, state_type, area, widget, detail, x1, x2, y)
-    VALUE self, gdkwindow, state_type, area, widget, detail, x1, x2, y;
+rg_paint_hline(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE area, VALUE widget, VALUE detail, VALUE x1, VALUE x2, VALUE y)
 {
     gtk_paint_hline(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2REC(area),
@@ -259,10 +231,7 @@ style_paint_hline(self, gdkwindow, state_type, area, widget, detail, x1, x2, y)
 }
 
 static VALUE
-style_paint_option(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                   x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_option(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_option(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                      RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -273,31 +242,30 @@ style_paint_option(self, gdkwindow, state_type, shadow_type, area, widget, detai
 }
 
 static VALUE
-style_paint_polygon(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                    points, fill)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    points, fill;
+rg_paint_polygon(VALUE self, VALUE rbwindow, VALUE rbstate_type,
+                    VALUE rbshadow_type, VALUE rbarea, VALUE rbwidget,
+                    VALUE rbdetail, VALUE rbpoints, VALUE rbfill)
 {
-    int i;
-    GdkPoint* gpoints = g_new (GdkPoint, RARRAY_LEN(points));
+    GtkStyle *style = _SELF(self);
+    GdkWindow *window = GDK_WINDOW(RVAL2GOBJ(rbwindow));
+    GtkStateType state_type = RVAL2STATE(rbstate_type);
+    GtkShadowType shadow_type = RVAL2SHADOW(rbshadow_type);
+    GdkRectangle *area = RVAL2REC(rbarea);
+    GtkWidget *widget = GTK_WIDGET(RVAL2GOBJ(rbwidget));
+    const gchar *detail = RVAL2CSTR_ACCEPT_NIL(rbdetail);
+    gboolean fill = RVAL2CBOOL(rbfill);
+    long n;
+    GdkPoint *points = RVAL2GDKPOINTS(rbpoints, &n);
+
+    gtk_paint_polygon(style, window, state_type, shadow_type, area, widget, detail, points, n, fill);
+
+    g_free(points);
 
-    for (i = 0; i < RARRAY_LEN(points); i++){
-        gpoints[i].x = RARRAY_PTR(RARRAY_PTR(points)[i])[0];
-        gpoints[i].y = RARRAY_PTR(RARRAY_PTR(points)[i])[1];
-    }
-    gtk_paint_polygon(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
-                      RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
-                      GTK_WIDGET(RVAL2GOBJ(widget)), 
-                      NIL_P(detail) ? NULL : RVAL2CSTR(detail), gpoints,
-                      RARRAY_LEN(points), RVAL2CBOOL(fill));
     return self;
 }
 
 static VALUE
-style_paint_shadow(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                   x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_shadow(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_shadow(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                      RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -308,10 +276,7 @@ style_paint_shadow(self, gdkwindow, state_type, shadow_type, area, widget, detai
 }
 
 static VALUE
-style_paint_shadow_gap(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                  x, y, width, height, gap_side, gap_x, gap_width)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height, gap_side, gap_x, gap_width;
+rg_paint_shadow_gap(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height, VALUE gap_side, VALUE gap_x, VALUE gap_width)
 {
     gtk_paint_shadow_gap(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                          RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -323,10 +288,7 @@ style_paint_shadow_gap(self, gdkwindow, state_type, shadow_type, area, widget, d
 }
 
 static VALUE
-style_paint_slider(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                   x, y, width, height, orientation)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height, orientation;
+rg_paint_slider(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height, VALUE orientation)
 {
     gtk_paint_slider(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                      RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -338,10 +300,7 @@ style_paint_slider(self, gdkwindow, state_type, shadow_type, area, widget, detai
 }
 
 static VALUE
-style_paint_tab(self, gdkwindow, state_type, shadow_type, area, widget, detail,
-                x, y, width, height)
-    VALUE self, gdkwindow, state_type, shadow_type, area, widget, detail,
-    x, y, width, height;
+rg_paint_tab(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE shadow_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_tab(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                   RVAL2STATE(state_type), RVAL2SHADOW(shadow_type), RVAL2REC(area),
@@ -352,8 +311,7 @@ style_paint_tab(self, gdkwindow, state_type, shadow_type, area, widget, detail,
 }
 
 static VALUE
-style_paint_vline(self, gdkwindow, state_type, area, widget, detail, y1, y2, x)
-    VALUE self, gdkwindow, state_type, area, widget, detail, y1, y2, x;
+rg_paint_vline(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE area, VALUE widget, VALUE detail, VALUE y1, VALUE y2, VALUE x)
 {
     gtk_paint_vline(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                     RVAL2STATE(state_type), RVAL2REC(area),
@@ -364,10 +322,7 @@ style_paint_vline(self, gdkwindow, state_type, area, widget, detail, y1, y2, x)
 }
 
 static VALUE
-style_paint_expander(self, gdkwindow, state_type, area, widget, detail,
-                     x, y, expander_style)
-    VALUE self, gdkwindow, state_type, area, widget, detail,
-    x, y, expander_style;
+rg_paint_expander(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE expander_style)
 {
     gtk_paint_expander(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                        RVAL2STATE(state_type), RVAL2REC(area),
@@ -378,10 +333,7 @@ style_paint_expander(self, gdkwindow, state_type, area, widget, detail,
 }
 
 static VALUE
-style_paint_layout(self, gdkwindow, state_type, use_text, area, 
-                   widget, detail, x, y, layout)
-    VALUE self, gdkwindow, state_type, use_text, area, widget, detail,
-    x, y, layout;
+rg_paint_layout(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE use_text, VALUE area, VALUE widget, VALUE detail, VALUE x, VALUE y, VALUE layout)
 {
     gtk_paint_layout(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                      RVAL2STATE(state_type), RVAL2CBOOL(use_text), RVAL2REC(area),
@@ -392,10 +344,7 @@ style_paint_layout(self, gdkwindow, state_type, use_text, area,
 }
 
 static VALUE
-style_paint_resize_grip(self, gdkwindow, state_type, area, widget, detail,
-                        edge, x, y, width, height)
-    VALUE self, gdkwindow, state_type, area, widget, detail, edge, x, y, 
-    width, height;
+rg_paint_resize_grip(VALUE self, VALUE gdkwindow, VALUE state_type, VALUE area, VALUE widget, VALUE detail, VALUE edge, VALUE x, VALUE y, VALUE width, VALUE height)
 {
     gtk_paint_resize_grip(_SELF(self), GDK_WINDOW(RVAL2GOBJ(gdkwindow)),
                           RVAL2STATE(state_type), RVAL2REC(area),
@@ -408,8 +357,7 @@ style_paint_resize_grip(self, gdkwindow, state_type, area, widget, detail,
 }
 #if GTK_CHECK_VERSION(2,4,0)
 static VALUE
-style_s_draw_insertion_cursor(self, widget, drawable, area, location, is_primary, direction, draw_arrow)
-    VALUE self, widget, drawable, area, location, is_primary, direction, draw_arrow;
+rg_s_draw_insertion_cursor(VALUE self, VALUE widget, VALUE drawable, VALUE area, VALUE location, VALUE is_primary, VALUE direction, VALUE draw_arrow)
 {
     gtk_draw_insertion_cursor(GTK_WIDGET(RVAL2GOBJ(widget)),
                               GDK_WINDOW(RVAL2GOBJ(drawable)),
@@ -423,8 +371,7 @@ style_s_draw_insertion_cursor(self, widget, drawable, area, location, is_primary
 
 #define DEFINE_STYLE_COLOR(type) \
 static VALUE \
-style_set_ ## type(self, idx, r, g, b) \
-    VALUE self, idx, r, g, b; \
+style_set_ ## type(VALUE self, VALUE idx, VALUE r, VALUE g, VALUE b) \
 { \
   GdkColor *color; \
   int i = NUM2INT(idx); \
@@ -437,8 +384,7 @@ style_set_ ## type(self, idx, r, g, b) \
   return self; \
 } \
 static VALUE \
-style_ ## type(self, idx) \
-    VALUE self, idx; \
+style_ ## type(VALUE self, VALUE idx) \
 { \
   int i = NUM2INT(idx); \
  \
@@ -456,34 +402,30 @@ DEFINE_STYLE_COLOR(base);
 DEFINE_STYLE_COLOR(text_aa);
 
 static VALUE
-style_black(self)
-    VALUE self;
+rg_black(VALUE self)
 {
     return BOXED2RVAL(&_SELF(self)->black, GDK_TYPE_COLOR);
 }
 
 static VALUE
-style_set_black(self, r, g, b)
-    VALUE self, r, g, b;
+rg_set_black(VALUE self, VALUE r, VALUE g, VALUE b)
 {
    GdkColor* color =  &(_SELF(self)->black);
    color->red   = NUM2INT(r); 
    color->green = NUM2INT(g); 
    color->blue  = NUM2INT(b); 
-   
+
    return self;
 }
 
 static VALUE
-style_white(self)
-    VALUE self;
+rg_white(VALUE self)
 {
     return BOXED2RVAL(&_SELF(self)->white, GDK_TYPE_COLOR);
 }
 
 static VALUE
-style_set_white(self, r, g, b)
-    VALUE self, r, g, b;
+rg_set_white(VALUE self, VALUE r, VALUE g, VALUE b)
 {
     GdkColor* color =  &(_SELF(self)->white);
     color->red   = NUM2INT(r); 
@@ -493,16 +435,14 @@ style_set_white(self, r, g, b)
 }
 
 static VALUE
-style_font_desc(self)
-    VALUE self;
+rg_font_desc(VALUE self)
 {
     PangoFontDescription* desc = _SELF(self)->font_desc;
     return BOXED2RVAL(desc, PANGO_TYPE_FONT_DESCRIPTION);
 }
 
 static VALUE
-style_set_font_desc(self, font_desc)
-    VALUE self, font_desc;
+rg_set_font_desc(VALUE self, VALUE font_desc)
 {
     GtkStyle *style = _SELF(self);
     if (style->font_desc)
@@ -512,30 +452,26 @@ style_set_font_desc(self, font_desc)
 }
 
 static VALUE
-style_xthickness(self)
-    VALUE self;
+rg_xthickness(VALUE self)
 {
     return INT2NUM(_SELF(self)->xthickness);
 }
 
 static VALUE
-style_set_xthickness(self, xthickness)
-    VALUE self, xthickness;
+rg_set_xthickness(VALUE self, VALUE xthickness)
 {
     _SELF(self)->xthickness = NUM2INT(xthickness);
     return self;
 }
 
 static VALUE
-style_ythickness(self)
-    VALUE self;
+rg_ythickness(VALUE self)
 {
     return INT2NUM(_SELF(self)->ythickness);
 }
 
 static VALUE
-style_set_ythickness(self, ythickness)
-    VALUE self, ythickness;
+rg_set_ythickness(VALUE self, VALUE ythickness)
 {
     _SELF(self)->ythickness = NUM2INT(ythickness);
     return self;
@@ -543,8 +479,7 @@ style_set_ythickness(self, ythickness)
 
 #define DEFINE_STYLE_GC(type) \
 static VALUE \
-style_set_ ## type(self, idx, gc) \
-    VALUE self, idx, gc; \
+style_set_ ## type(VALUE self, VALUE idx, VALUE gc) \
 { \
   int i = NUM2INT(idx); \
  \
@@ -553,8 +488,7 @@ style_set_ ## type(self, idx, gc) \
   return self;\
 } \
 static VALUE \
-style_ ## type(self, idx) \
-    VALUE self, idx; \
+style_ ## type(VALUE self, VALUE idx) \
 { \
   int i = NUM2INT(idx); \
  \
@@ -572,38 +506,33 @@ DEFINE_STYLE_GC(base_gc);
 DEFINE_STYLE_GC(text_aa_gc);
 
 static VALUE
-style_black_gc(self)
-    VALUE self;
+rg_black_gc(VALUE self)
 {
     return GOBJ2RVAL(_SELF(self)->black_gc);
 }
 
 static VALUE
-style_set_black_gc(self, gc)
-    VALUE self, gc;
+rg_set_black_gc(VALUE self, VALUE gc)
 {
     _SELF(self)->black_gc = GDK_GC(RVAL2GOBJ(gc));
     return self;
 }
 
 static VALUE
-style_white_gc(self)
-    VALUE self;
+rg_white_gc(VALUE self)
 {
     return GOBJ2RVAL(_SELF(self)->white_gc);
 }
 
 static VALUE
-style_set_white_gc(self, gc)
-    VALUE self, gc;
+rg_set_white_gc(VALUE self, VALUE gc)
 {
     _SELF(self)->white_gc = GDK_GC(RVAL2GOBJ(gc));
     return self;
 }
 
 static VALUE
-style_bg_pixmap(self, idx)
-    VALUE self, idx;
+rg_bg_pixmap(VALUE self, VALUE idx)
 {
     int i = NUM2INT(idx);
 
@@ -613,8 +542,7 @@ style_bg_pixmap(self, idx)
 }
 
 static VALUE
-style_set_bg_pixmap(self, idx, bg_pixmap)
-    VALUE self, idx, bg_pixmap;
+rg_set_bg_pixmap(VALUE self, VALUE idx, VALUE bg_pixmap)
 {
     int i = NUM2INT(idx);
     GdkPixmap* pixmap = RVAL2GOBJ(bg_pixmap);
@@ -626,97 +554,97 @@ style_set_bg_pixmap(self, idx, bg_pixmap)
 }
 
 void 
-Init_gtk_style()
+Init_gtk_style(VALUE mGtk)
 {
-    VALUE gStyle = G_DEF_CLASS(GTK_TYPE_STYLE, "Style", mGtk);
+    VALUE RG_TARGET_NAMESPACE = G_DEF_CLASS(GTK_TYPE_STYLE, "Style", mGtk);
 
-    rb_define_method(gStyle, "initialize", style_initialize, 0);
-    rb_define_method(gStyle, "attached?", style_attached, 0);
-    rb_define_method(gStyle, "copy", style_copy, 0);
-    rb_define_method(gStyle, "clone", style_copy, 0);
-    rb_define_method(gStyle, "dup", style_copy, 0);
-    rb_define_method(gStyle, "attach", style_attach, 1);
-    rb_define_method(gStyle, "detach", style_detach, 0);
-    rb_define_method(gStyle, "set_background", style_set_background, 2);
-    rb_define_method(gStyle, "apply_default_background", style_apply_default_background, 8);
+    RG_DEF_METHOD(initialize, 0);
+    RG_DEF_METHOD_P(attached, 0);
+    RG_DEF_METHOD(copy, 0);
+    RG_DEF_ALIAS("clone", "copy");
+    RG_DEF_ALIAS("dup", "copy");
+    RG_DEF_METHOD(attach, 1);
+    RG_DEF_METHOD(detach, 0);
+    RG_DEF_METHOD(set_background, 2);
+    RG_DEF_METHOD(apply_default_background, 8);
 #if GTK_CHECK_VERSION(2,10,0)
-    rb_define_method(gStyle, "lookup_color", style_lookup_color, 1);
+    RG_DEF_METHOD(lookup_color, 1);
 #endif
-    rb_define_method(gStyle, "lookup_icon_set", style_lookup_icon_set, 1);
-    rb_define_method(gStyle, "render_icon", style_render_icon, 6);
-    rb_define_method(gStyle, "paint_arrow", style_paint_arrow, 12);
-    rb_define_method(gStyle, "paint_box", style_paint_box, 10);
-    rb_define_method(gStyle, "paint_box_gap", style_paint_box_gap, 13);
-    rb_define_method(gStyle, "paint_check", style_paint_check, 10);
-    rb_define_method(gStyle, "paint_diamond", style_paint_diamond, 10);
-    rb_define_method(gStyle, "paint_extension", style_paint_extension, 10);
-    rb_define_method(gStyle, "paint_flat_box", style_paint_flat_box, 10);
-    rb_define_method(gStyle, "paint_focus", style_paint_focus, 9);
-    rb_define_method(gStyle, "paint_handle", style_paint_handle, 11);
-    rb_define_method(gStyle, "paint_hline", style_paint_hline, 8);
-    rb_define_method(gStyle, "paint_option", style_paint_option, 10);
-    rb_define_method(gStyle, "paint_polygon", style_paint_polygon, 8);
-    rb_define_method(gStyle, "paint_shadow", style_paint_shadow, 10);
-    rb_define_method(gStyle, "paint_shadow_gap", style_paint_shadow_gap, 13);
-    rb_define_method(gStyle, "paint_slider", style_paint_slider, 11);
-    rb_define_method(gStyle, "paint_tab", style_paint_tab, 10);
-    rb_define_method(gStyle, "paint_vline", style_paint_vline, 9);
-    rb_define_method(gStyle, "paint_expander", style_paint_expander, 8);
-    rb_define_method(gStyle, "paint_layout", style_paint_layout, 9);
-    rb_define_method(gStyle, "paint_resize_grip", style_paint_resize_grip, 10);
+    RG_DEF_METHOD(lookup_icon_set, 1);
+    RG_DEF_METHOD(render_icon, 6);
+    RG_DEF_METHOD(paint_arrow, 12);
+    RG_DEF_METHOD(paint_box, 10);
+    RG_DEF_METHOD(paint_box_gap, 13);
+    RG_DEF_METHOD(paint_check, 10);
+    RG_DEF_METHOD(paint_diamond, 10);
+    RG_DEF_METHOD(paint_extension, 10);
+    RG_DEF_METHOD(paint_flat_box, 10);
+    RG_DEF_METHOD(paint_focus, 9);
+    RG_DEF_METHOD(paint_handle, 11);
+    RG_DEF_METHOD(paint_hline, 8);
+    RG_DEF_METHOD(paint_option, 10);
+    RG_DEF_METHOD(paint_polygon, 8);
+    RG_DEF_METHOD(paint_shadow, 10);
+    RG_DEF_METHOD(paint_shadow_gap, 13);
+    RG_DEF_METHOD(paint_slider, 11);
+    RG_DEF_METHOD(paint_tab, 10);
+    RG_DEF_METHOD(paint_vline, 9);
+    RG_DEF_METHOD(paint_expander, 8);
+    RG_DEF_METHOD(paint_layout, 9);
+    RG_DEF_METHOD(paint_resize_grip, 10);
 #if GTK_CHECK_VERSION(2,4,0)
-    rb_define_singleton_method(gStyle, "draw_insertion_cursor", style_s_draw_insertion_cursor, 7);
+    RG_DEF_SMETHOD(draw_insertion_cursor, 7);
 #endif
-    rb_define_method(gStyle, "fg", style_fg, 1);
-    rb_define_method(gStyle, "bg", style_bg, 1);
-    rb_define_method(gStyle, "light", style_light, 1);
-    rb_define_method(gStyle, "dark", style_dark, 1);
-    rb_define_method(gStyle, "mid", style_mid, 1);
-    rb_define_method(gStyle, "text", style_text, 1);
-    rb_define_method(gStyle, "base", style_base, 1);
-    rb_define_method(gStyle, "text_aa", style_text_aa, 1);
-    rb_define_method(gStyle, "set_fg", style_set_fg, 4);
-    rb_define_method(gStyle, "set_bg", style_set_bg, 4);
-    rb_define_method(gStyle, "set_light", style_set_light, 4);
-    rb_define_method(gStyle, "set_dark", style_set_dark, 4);
-    rb_define_method(gStyle, "set_mid", style_set_mid, 4);
-    rb_define_method(gStyle, "set_text", style_set_text, 4);
-    rb_define_method(gStyle, "set_base", style_set_base, 4);
-    rb_define_method(gStyle, "set_text_aa", style_set_text_aa, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "fg", style_fg, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "bg", style_bg, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "light", style_light, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "dark", style_dark, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "mid", style_mid, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "text", style_text, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "base", style_base, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "text_aa", style_text_aa, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_fg", style_set_fg, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_bg", style_set_bg, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_light", style_set_light, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_dark", style_set_dark, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_mid", style_set_mid, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_text", style_set_text, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_base", style_set_base, 4);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_text_aa", style_set_text_aa, 4);
 
-    rb_define_method(gStyle, "black", style_black, 0);
-    rb_define_method(gStyle, "set_black", style_set_black, 3);
-    rb_define_method(gStyle, "white", style_white, 0);
-    rb_define_method(gStyle, "set_white", style_set_white, 3);
-    rb_define_method(gStyle, "font_desc", style_font_desc, 0);
-    rb_define_method(gStyle, "set_font_desc", style_set_font_desc, 1);
-    rb_define_method(gStyle, "xthickness", style_xthickness, 0);
-    rb_define_method(gStyle, "ythickness", style_ythickness, 0);
-    rb_define_method(gStyle, "set_xthickness", style_set_xthickness, 1);
-    rb_define_method(gStyle, "set_ythickness", style_set_ythickness, 1);
+    RG_DEF_METHOD(black, 0);
+    RG_DEF_METHOD(set_black, 3);
+    RG_DEF_METHOD(white, 0);
+    RG_DEF_METHOD(set_white, 3);
+    RG_DEF_METHOD(font_desc, 0);
+    RG_DEF_METHOD(set_font_desc, 1);
+    RG_DEF_METHOD(xthickness, 0);
+    RG_DEF_METHOD(ythickness, 0);
+    RG_DEF_METHOD(set_xthickness, 1);
+    RG_DEF_METHOD(set_ythickness, 1);
 
-    rb_define_method(gStyle, "fg_gc", style_fg_gc, 1);
-    rb_define_method(gStyle, "bg_gc", style_bg_gc, 1);
-    rb_define_method(gStyle, "light_gc", style_light_gc, 1);
-    rb_define_method(gStyle, "dark_gc", style_dark_gc, 1);
-    rb_define_method(gStyle, "mid_gc", style_mid_gc, 1);
-    rb_define_method(gStyle, "text_gc", style_text_gc, 1);
-    rb_define_method(gStyle, "base_gc", style_base_gc, 1);
-    rb_define_method(gStyle, "text_aa_gc", style_text_aa_gc, 1);
-    rb_define_method(gStyle, "black_gc", style_black_gc, 0);
-    rb_define_method(gStyle, "white_gc", style_white_gc, 0);
-    rb_define_method(gStyle, "bg_pixmap", style_bg_pixmap, 1);
-    rb_define_method(gStyle, "set_fg_gc", style_set_fg_gc, 2);
-    rb_define_method(gStyle, "set_bg_gc", style_set_bg_gc, 2);
-    rb_define_method(gStyle, "set_light_gc", style_set_light_gc, 2);
-    rb_define_method(gStyle, "set_dark_gc", style_set_dark_gc, 2);
-    rb_define_method(gStyle, "set_mid_gc", style_set_mid_gc, 2);
-    rb_define_method(gStyle, "set_text_gc", style_set_text_gc, 2);
-    rb_define_method(gStyle, "set_base_gc", style_set_base_gc, 2);
-    rb_define_method(gStyle, "set_text_aa_gc", style_set_text_aa_gc, 2);
-    rb_define_method(gStyle, "set_black_gc", style_set_black_gc, 1);
-    rb_define_method(gStyle, "set_white_gc", style_set_white_gc, 1);
-    rb_define_method(gStyle, "set_bg_pixmap", style_set_bg_pixmap, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "fg_gc", style_fg_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "bg_gc", style_bg_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "light_gc", style_light_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "dark_gc", style_dark_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "mid_gc", style_mid_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "text_gc", style_text_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "base_gc", style_base_gc, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "text_aa_gc", style_text_aa_gc, 1);
+    RG_DEF_METHOD(black_gc, 0);
+    RG_DEF_METHOD(white_gc, 0);
+    RG_DEF_METHOD(bg_pixmap, 1);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_fg_gc", style_set_fg_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_bg_gc", style_set_bg_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_light_gc", style_set_light_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_dark_gc", style_set_dark_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_mid_gc", style_set_mid_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_text_gc", style_set_text_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_base_gc", style_set_base_gc, 2);
+    rb_define_method(RG_TARGET_NAMESPACE, "set_text_aa_gc", style_set_text_aa_gc, 2);
+    RG_DEF_METHOD(set_black_gc, 1);
+    RG_DEF_METHOD(set_white_gc, 1);
+    RG_DEF_METHOD(set_bg_pixmap, 2);
 
-    G_DEF_SETTERS(gStyle);
+    G_DEF_SETTERS(RG_TARGET_NAMESPACE);
 }
