@@ -1,17 +1,28 @@
 /* -*- c-file-style: "ruby"; indent-tabs-mode: nil -*- */
-/************************************************
-
-  rbgtktextbuffer.c -
-
-  $Author: ggc $
-  $Date: 2007/07/13 16:07:32 $
-
-  Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
-  Copyright (C) 2002,2003 Masahiro Sakai
-************************************************/
+/*
+ *  Copyright (C) 2011  Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002-2005 Ruby-GNOME2 Project Team
+ *  Copyright (C) 2002,2003 Masahiro Sakai
+ *
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2.1 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA  02110-1301  USA
+ */
 
 #include "global.h"
 
+#define RG_TARGET_NAMESPACE cTextBuffer
 #define _SELF(s) (GTK_TEXT_BUFFER(RVAL2GOBJ(s)))
 #define RVAL2ITR(i) (GtkTextIter*)RVAL2BOXED(i, GTK_TYPE_TEXT_ITER)
 #define ITR2RVAL(i) (BOXED2RVAL(i, GTK_TYPE_TEXT_ITER))
@@ -24,10 +35,7 @@
 static ID id_tagtable;
 
 static VALUE
-txt_initialize(argc, argv, self)
-    int argc;
-    VALUE* argv;
-    VALUE  self;
+rg_initialize(int argc, VALUE *argv, VALUE self)
 {
     VALUE table;
     rb_scan_args(argc, argv, "01", &table);
@@ -41,39 +49,28 @@ txt_initialize(argc, argv, self)
 }
 
 static VALUE
-txt_get_line_count(self)
-    VALUE self;
+rg_line_count(VALUE self)
 {
     return INT2NUM(gtk_text_buffer_get_line_count(_SELF(self)));
 }
 
 static VALUE
-txt_get_char_count(self)
-    VALUE self;
+rg_char_count(VALUE self)
 {
     return INT2NUM(gtk_text_buffer_get_char_count(_SELF(self)));
 }
 
 static VALUE
-txt_get_tag_table(self)
-    VALUE self;
-{
-    return GOBJ2RVAL(gtk_text_buffer_get_tag_table(_SELF(self)));
-}
-
-static VALUE
-txt_set_text(self, text)
-    VALUE self, text;
+txt_set_text(VALUE self, VALUE text)
 {
     StringValue(text);
-    gtk_text_buffer_set_text(_SELF(self), RVAL2CSTR(text), RSTRING_LEN(text));
+    gtk_text_buffer_set_text(_SELF(self), RSTRING_PTR(text), RSTRING_LEN(text));
     return self;
 }
 
 #if GTK_CHECK_VERSION(2,6,0)
 static VALUE
-txt_backspace(self, iter, interactive, default_editable)
-    VALUE self, iter, interactive, default_editable;
+rg_backspace(VALUE self, VALUE iter, VALUE interactive, VALUE default_editable)
 {
     return CBOOL2RVAL(gtk_text_buffer_backspace(_SELF(self), RVAL2ITR(iter),
                                                 RVAL2CBOOL(interactive),
@@ -82,39 +79,38 @@ txt_backspace(self, iter, interactive, default_editable)
 #endif
 
 static VALUE
-txt_insert_at_cursor(self, text)
-    VALUE self, text;
+rg_insert_at_cursor(VALUE self, VALUE text)
 {
     StringValue(text);
-    gtk_text_buffer_insert_at_cursor(_SELF(self),
-                                     RVAL2CSTR(text), RSTRING_LEN(text));
+    gtk_text_buffer_insert_at_cursor(_SELF(self), RSTRING_PTR(text), RSTRING_LEN(text));
     return self;
 }
 
 static VALUE
-txt_insert_interactive(self, iter, text, editable)
-    VALUE self, iter, text, editable;
+rg_insert_interactive(VALUE self, VALUE iter, VALUE text, VALUE editable)
 {
+    StringValue(text);
+
     return CBOOL2RVAL(gtk_text_buffer_insert_interactive(_SELF(self),
                                                          RVAL2ITR(iter),
-                                                         RVAL2CSTR(text),
+                                                         RSTRING_PTR(text),
                                                          RSTRING_LEN(text),
                                                          RVAL2CBOOL(editable)));
 }
 
 static VALUE
-txt_insert_interactive_at_cursor(self, text, editable)
-    VALUE self, text, editable;
+rg_insert_interactive_at_cursor(VALUE self, VALUE text, VALUE editable)
 {
+    StringValue(text);
+
     return CBOOL2RVAL(gtk_text_buffer_insert_interactive_at_cursor(_SELF(self),
-                                                                   RVAL2CSTR(text),
+                                                                   RSTRING_PTR(text),
                                                                    RSTRING_LEN(text),
                                                                    RVAL2CBOOL(editable)));
 }
 
 static VALUE
-txt_insert_range(self, iter, start, end)
-    VALUE self, iter, start, end;
+rg_insert_range(VALUE self, VALUE iter, VALUE start, VALUE end)
 {
     gtk_text_buffer_insert_range(_SELF(self), RVAL2ITR(iter),
                                  RVAL2ITR(start), RVAL2ITR(end));
@@ -122,8 +118,7 @@ txt_insert_range(self, iter, start, end)
 }
 
 static VALUE
-txt_insert_range_interactive(self, iter, start, end, editable)
-    VALUE self, iter, start, end, editable;
+rg_insert_range_interactive(VALUE self, VALUE iter, VALUE start, VALUE end, VALUE editable)
 {
     return CBOOL2RVAL(gtk_text_buffer_insert_range_interactive(_SELF(self),
                                                                RVAL2ITR(iter),
@@ -133,8 +128,7 @@ txt_insert_range_interactive(self, iter, start, end, editable)
 }
 
 static VALUE
-txt_delete(self, start, end)
-    VALUE self, start, end;
+rg_delete(VALUE self, VALUE start, VALUE end)
 {
     gtk_text_buffer_delete(GTK_TEXT_BUFFER(RVAL2GOBJ(self)),
                            RVAL2ITR(start), RVAL2ITR(end));
@@ -142,8 +136,7 @@ txt_delete(self, start, end)
 }
 
 static VALUE
-txt_delete_interactive(self, start, end, editable)
-    VALUE self, start, end, editable;
+rg_delete_interactive(VALUE self, VALUE start, VALUE end, VALUE editable)
 {
     return CBOOL2RVAL(gtk_text_buffer_delete_interactive(_SELF(self),
                                                          RVAL2ITR(start),
@@ -152,10 +145,7 @@ txt_delete_interactive(self, start, end, editable)
 }
 
 static VALUE
-txt_get_text(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+rg_get_text(int argc, VALUE *argv, VALUE self)
 {
     VALUE start, end, include_hidden_chars;
     GtkTextIter start_iter, end_iter;
@@ -181,17 +171,13 @@ txt_get_text(argc, argv, self)
 }
 
 static VALUE
-txt_get_text_all(self)
-    VALUE self;
+txt_get_text_all(VALUE self)
 {
-    return txt_get_text(0, NULL, self);
+    return rg_get_text(0, NULL, self);
 }
 
 static VALUE
-txt_get_slice(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+rg_get_slice(int argc, VALUE *argv, VALUE self)
 {
     VALUE start, end, include_hidden_chars;
     GtkTextIter start_iter, end_iter;
@@ -217,15 +203,13 @@ txt_get_slice(argc, argv, self)
 }
 
 static VALUE
-txt_get_slice_all(self)
-    VALUE self;
+rg_slice(VALUE self)
 {
-    return txt_get_slice(0, NULL, self);
+    return rg_get_slice(0, NULL, self);
 }
 
 static VALUE
-txt_insert_pixbuf(self, iter, pixbuf)
-    VALUE self, iter, pixbuf;
+rg_insert_pixbuf(VALUE self, VALUE iter, VALUE pixbuf)
 {
     G_CHILD_ADD(self, iter);
     G_CHILD_ADD(iter, pixbuf);
@@ -238,8 +222,7 @@ txt_insert_pixbuf(self, iter, pixbuf)
 }
 
 static VALUE
-txt_insert_child_anchor(self, iter, anchor)
-    VALUE self, iter, anchor;
+rg_insert_child_anchor(VALUE self, VALUE iter, VALUE anchor)
 {
     G_CHILD_ADD(self, iter);
     G_CHILD_ADD(iter, anchor);
@@ -251,8 +234,7 @@ txt_insert_child_anchor(self, iter, anchor)
 }
 
 static VALUE
-txt_create_child_anchor(self, iter)
-    VALUE self, iter;
+rg_create_child_anchor(VALUE self, VALUE iter)
 {
     VALUE ret = GOBJ2RVAL(gtk_text_buffer_create_child_anchor(_SELF(self), RVAL2ITR(iter)));
     G_CHILD_ADD(self, ret);
@@ -260,8 +242,7 @@ txt_create_child_anchor(self, iter)
 }
 
 static VALUE
-txt_create_mark(self, name, where, left_gravity)
-    VALUE self, name, where, left_gravity;
+rg_create_mark(VALUE self, VALUE name, VALUE where, VALUE left_gravity)
 {
     VALUE ret = GOBJ2RVAL(gtk_text_buffer_create_mark(_SELF(self),
                                                       NIL_P(name) ? NULL : RVAL2CSTR(name),
@@ -273,8 +254,7 @@ txt_create_mark(self, name, where, left_gravity)
 
 #if GTK_CHECK_VERSION(2,12,0)
 static VALUE
-txt_add_mark(self, mark, where)
-    VALUE self, mark, where;
+rg_add_mark(VALUE self, VALUE mark, VALUE where)
 {
     gtk_text_buffer_add_mark(_SELF(self), RVAL2MARK(mark), RVAL2ITR(where));
     return self;
@@ -282,8 +262,7 @@ txt_add_mark(self, mark, where)
 #endif
 
 static VALUE
-txt_delete_mark(self, mark)
-    VALUE self, mark;
+rg_delete_mark(VALUE self, VALUE mark)
 {
     if (rb_obj_is_kind_of(mark, GTYPE2CLASS(GTK_TYPE_TEXT_MARK))){
         G_CHILD_REMOVE(self, mark);
@@ -296,31 +275,27 @@ txt_delete_mark(self, mark)
 }
 
 static VALUE
-txt_get_mark(self, name)
-    VALUE self, name;
+rg_get_mark(VALUE self, VALUE name)
 {
     return GOBJ2RVAL(gtk_text_buffer_get_mark(_SELF(self), RVAL2CSTR(name)));
 }
 
 /*
 static VALUE
-txt_get_insert(self)
-    VALUE self;
+rg_get_insert(VALUE self)
 {
     return GOBJ2RVAL(gtk_text_buffer_get_insert(_SELF(self)));
 }
 */
 
 static VALUE
-txt_get_selection_bound(self)
-    VALUE self;
+rg_selection_bound(VALUE self)
 {
     return GOBJ2RVAL(gtk_text_buffer_get_selection_bound(_SELF(self)));
 }
 
 static VALUE
-txt_place_cursor(self, where)
-    VALUE self, where;
+rg_place_cursor(VALUE self, VALUE where)
 {
     gtk_text_buffer_place_cursor(_SELF(self), RVAL2ITR(where));
     return self;
@@ -328,8 +303,7 @@ txt_place_cursor(self, where)
 
 #if GTK_CHECK_VERSION(2,4,0)
 static VALUE
-txt_select_range(self, ins, bound)
-    VALUE self, ins, bound;
+rg_select_range(VALUE self, VALUE ins, VALUE bound)
 {
     gtk_text_buffer_select_range(_SELF(self), RVAL2ITR(ins), RVAL2ITR(bound));
     return self;
@@ -337,8 +311,7 @@ txt_select_range(self, ins, bound)
 #endif
 
 static VALUE
-txt_get_iter_at_child_anchor(self, anchor)
-    VALUE self, anchor;
+rg_get_iter_at_child_anchor(VALUE self, VALUE anchor)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_iter_at_child_anchor(_SELF(self), &iter, RVAL2ANCHOR(anchor));
@@ -346,23 +319,20 @@ txt_get_iter_at_child_anchor(self, anchor)
 }
 
 static VALUE
-txt_get_modified(self)
-    VALUE self;
+rg_modified_p(VALUE self)
 {
     return CBOOL2RVAL(gtk_text_buffer_get_modified(_SELF(self)));
 }
 
 static VALUE
-txt_set_modified(self, setting)
-    VALUE self, setting;
+rg_set_modified(VALUE self, VALUE setting)
 {
     gtk_text_buffer_set_modified(_SELF(self), RVAL2CBOOL(setting));
     return setting;
 }
 
 static VALUE
-txt_add_selection_clipboard(self, clipboard)
-    VALUE self, clipboard;
+rg_add_selection_clipboard(VALUE self, VALUE clipboard)
 {
     G_CHILD_ADD(self, clipboard);
     gtk_text_buffer_add_selection_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard));
@@ -370,8 +340,7 @@ txt_add_selection_clipboard(self, clipboard)
 }
 
 static VALUE
-txt_remove_selection_clipboard(self, clipboard)
-    VALUE self, clipboard;
+rg_remove_selection_clipboard(VALUE self, VALUE clipboard)
 {
     G_CHILD_REMOVE(self, clipboard);
     gtk_text_buffer_remove_selection_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard));
@@ -380,13 +349,12 @@ txt_remove_selection_clipboard(self, clipboard)
 
 #if GTK_CHECK_VERSION(2,10,0)
 static VALUE
-txt_deserialize(self, content_buffer, format, iter, data)
-    VALUE self, content_buffer, format, iter, data;
+rg_deserialize(VALUE self, VALUE content_buffer, VALUE format, VALUE iter, VALUE data)
 {
     GError* error = NULL;
     gboolean ret;
 
-    Check_Type(data, T_STRING);
+    StringValue(data);
     ret = gtk_text_buffer_deserialize(_SELF(self), _SELF(content_buffer),
                                       RVAL2ATOM(format),
                                       RVAL2ITR(iter),
@@ -398,8 +366,7 @@ txt_deserialize(self, content_buffer, format, iter, data)
 }
 
 static VALUE
-txt_deserialize_get_can_create_tags(self, format)
-    VALUE self, format;
+rg_deserialize_can_create_tags_p(VALUE self, VALUE format)
 {
     return CBOOL2RVAL(gtk_text_buffer_deserialize_get_can_create_tags(_SELF(self), 
                                                                       RVAL2ATOM(format)));
@@ -407,8 +374,7 @@ txt_deserialize_get_can_create_tags(self, format)
 }
 
 static VALUE
-txt_deserialize_set_can_create_tags(self, format, can_create_tags)
-    VALUE self, format, can_create_tags;
+rg_deserialize_set_can_create_tags(VALUE self, VALUE format, VALUE can_create_tags)
 {
     gtk_text_buffer_deserialize_set_can_create_tags(_SELF(self),
                                                     RVAL2ATOM(format),
@@ -416,14 +382,8 @@ txt_deserialize_set_can_create_tags(self, format, can_create_tags)
     return self;
 }
 
-/* Defined as Properties
-GtkTargetList* gtk_text_buffer_get_copy_target_list
-                                            (GtkTextBuffer *buffer);
-*/
-
 static VALUE
-txt_get_deserialize_formats(self)
-    VALUE self;
+rg_deserialize_formats(VALUE self)
 {
     gint i;
     gint n_formats;
@@ -436,14 +396,8 @@ txt_get_deserialize_formats(self)
     return ary;
 }
 
-/* Defined as Properties
-GtkTargetList* gtk_text_buffer_get_paste_target_list
-                                            (GtkTextBuffer *buffer);
-*/
-
 static VALUE
-txt_get_serialize_formats(self)
-    VALUE self;
+rg_serialize_formats(VALUE self)
 {
     gint i;
     gint n_formats;
@@ -472,15 +426,14 @@ invoke_callback(VALUE data)
 }
 
 static gboolean
-deserialize_func(register_buffer, content_buffer, iter, data, length, create_tags, func, error)
-    GtkTextBuffer* register_buffer;
-    GtkTextBuffer* content_buffer;
-    GtkTextIter* iter;
-    const guint8 *data;
-    gsize length;
-    gboolean create_tags;
-    gpointer func;
-    GError** error;
+deserialize_func(GtkTextBuffer *register_buffer,
+                 GtkTextBuffer *content_buffer,
+                 GtkTextIter *iter,
+                 const guint8 *data,
+                 gsize length,
+                 gboolean create_tags,
+                 gpointer func,
+                 G_GNUC_UNUSED GError **error)
 {
     VALUE result;
     VALUE argv[5];
@@ -507,8 +460,7 @@ remove_callback_reference(gpointer callback)
 }
 
 static VALUE
-txt_register_deserialize_format(self, mime_type)
-    VALUE self, mime_type;
+rg_register_deserialize_format(VALUE self, VALUE mime_type)
 {
     VALUE block = rb_block_proc();
     GdkAtom atom;
@@ -522,21 +474,14 @@ txt_register_deserialize_format(self, mime_type)
 }
 
 static VALUE
-txt_register_deserialize_target(self, tagset_name)
-    VALUE self, tagset_name;
+rg_register_deserialize_target(VALUE self, VALUE tagset_name)
 {
     return ATOM2RVAL(gtk_text_buffer_register_deserialize_tagset(_SELF(self),
                                                                  (const gchar*)(NIL_P(tagset_name) ? NULL : RVAL2CSTR(tagset_name))));
 }
 
 static guint8*
-serialize_func(register_buffer, content_buffer, start, end, length, func)
-    GtkTextBuffer* register_buffer;
-    GtkTextBuffer* content_buffer;
-    GtkTextIter* start;
-    GtkTextIter* end;
-    gsize* length;
-    gpointer func;
+serialize_func(GtkTextBuffer *register_buffer, GtkTextBuffer *content_buffer, GtkTextIter *start, GtkTextIter *end, gsize *length, gpointer func)
 {
     VALUE result;
     VALUE argv[4];
@@ -553,14 +498,13 @@ serialize_func(register_buffer, content_buffer, start, end, length, func)
 
     /* This should return data as String */
     result = G_PROTECT_CALLBACK(invoke_callback, &arg);
-    Check_Type(result, T_STRING);
+    StringValue(result);
     *length = RSTRING_LEN(result);
     return (guint8*)(NIL_P(rb_errinfo()) ? RSTRING_PTR(result) : NULL);
 }
 
 static VALUE
-txt_register_serialize_format(self, mime_type)
-    VALUE self, mime_type;
+rg_register_serialize_format(VALUE self, VALUE mime_type)
 {
     VALUE block = rb_block_proc();
     GdkAtom atom;
@@ -574,19 +518,17 @@ txt_register_serialize_format(self, mime_type)
 }
 
 static VALUE
-txt_register_serialize_target(self, tagset_name)
-    VALUE self, tagset_name;
+rg_register_serialize_target(VALUE self, VALUE tagset_name)
 {
     return ATOM2RVAL(gtk_text_buffer_register_serialize_tagset(_SELF(self),
                                                                NIL_P(tagset_name) ? NULL : RVAL2CSTR(tagset_name)));
 }
 
 static VALUE
-txt_serialize(self, content_buffer, format, start, end)
-    VALUE self, content_buffer, format, start, end;
+rg_serialize(VALUE self, VALUE content_buffer, VALUE format, VALUE start, VALUE end)
 {
     gsize length;
-    guint8* ret = gtk_text_buffer_serialize(_SELF(self), _SELF(self),
+    guint8* ret = gtk_text_buffer_serialize(_SELF(self), _SELF(content_buffer),
                                             RVAL2ATOM(format),
                                             RVAL2ITR(start), RVAL2ITR(end),
                                             &length);
@@ -594,16 +536,14 @@ txt_serialize(self, content_buffer, format, start, end)
 }
 
 static VALUE
-txt_unregister_deserialize_format(self, format)
-    VALUE self, format;
+rg_unregister_deserialize_format(VALUE self, VALUE format)
 {
     gtk_text_buffer_unregister_deserialize_format(_SELF(self), RVAL2ATOM(format));
     return self;
 }
 
 static VALUE
-txt_unregister_serialize_format(self, format)
-    VALUE self, format;
+rg_unregister_serialize_format(VALUE self, VALUE format)
 {
     gtk_text_buffer_unregister_serialize_format(_SELF(self), RVAL2ATOM(format));
     return self;
@@ -611,8 +551,7 @@ txt_unregister_serialize_format(self, format)
 #endif
 
 static VALUE
-txt_cut_clipboard(self, clipboard, default_editable)
-    VALUE self, clipboard, default_editable;
+rg_cut_clipboard(VALUE self, VALUE clipboard, VALUE default_editable)
 {
     G_CHILD_ADD(self, clipboard);
     gtk_text_buffer_cut_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard), RVAL2CBOOL(default_editable));
@@ -620,8 +559,7 @@ txt_cut_clipboard(self, clipboard, default_editable)
 }
 
 static VALUE
-txt_copy_clipboard(self, clipboard)
-    VALUE self, clipboard;
+rg_copy_clipboard(VALUE self, VALUE clipboard)
 {
     G_CHILD_ADD(self, clipboard);
     gtk_text_buffer_copy_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard));
@@ -629,8 +567,7 @@ txt_copy_clipboard(self, clipboard)
 }
 
 static VALUE
-txt_paste_clipboard(self, clipboard, location, default_editable)
-    VALUE self, clipboard, location, default_editable;
+rg_paste_clipboard(VALUE self, VALUE clipboard, VALUE location, VALUE default_editable)
 {
     G_CHILD_ADD(self, clipboard);
     gtk_text_buffer_paste_clipboard(_SELF(self), RVAL2CLIPBOARD(clipboard),
@@ -640,25 +577,16 @@ txt_paste_clipboard(self, clipboard, location, default_editable)
 }
 
 static VALUE
-txt_get_selection_bounds(self)
-    VALUE self;
+rg_selection_bounds(VALUE self)
 {
     GtkTextIter start, end;
-    
+
     gboolean ret = gtk_text_buffer_get_selection_bounds(_SELF(self), &start, &end);
     return rb_ary_new3(3, ITR2RVAL(&start), ITR2RVAL(&end), CBOOL2RVAL(ret));
 }
 
-/* Defined as a proprety
-gboolean    gtk_text_buffer_get_has_selection
-                                            (GtkTextBuffer *buffer);
-*/
-
 static VALUE
-txt_delete_selection(argc, argv, self)
-    int argc;
-    VALUE* argv;
-    VALUE  self;
+rg_delete_selection(int argc, VALUE *argv, VALUE self)
 {
     VALUE interactive, default_editable;
     rb_scan_args(argc, argv, "20", &interactive, &default_editable); 
@@ -667,26 +595,23 @@ txt_delete_selection(argc, argv, self)
 }
 
 static VALUE
-txt_end_user_action(self)
-    VALUE self;
+rg_end_user_action(VALUE self)
 {
     gtk_text_buffer_end_user_action(_SELF(self));
     return self;
 }
 
 static VALUE
-txt_begin_user_action(self)
-    VALUE self;
+rg_begin_user_action(VALUE self)
 {
     gtk_text_buffer_begin_user_action(_SELF(self));
     if (rb_block_given_p())
-        rb_ensure(rb_yield, self, txt_end_user_action, self);
+        rb_ensure(rb_yield, self, rg_end_user_action, self);
     return self;
 }
 
 static VALUE
-txt_get_start_iter(self)
-    VALUE self;
+rg_start_iter(VALUE self)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_start_iter(_SELF(self), &iter);
@@ -694,8 +619,7 @@ txt_get_start_iter(self)
 }
 
 static VALUE
-txt_get_end_iter(self)
-    VALUE self;
+rg_end_iter(VALUE self)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_end_iter(_SELF(self), &iter);
@@ -703,8 +627,7 @@ txt_get_end_iter(self)
 }
 
 static VALUE
-txt_move_mark(self, mark, where)
-    VALUE self, mark, where;
+rg_move_mark(VALUE self, VALUE mark, VALUE where)
 {
     if (rb_obj_is_kind_of(mark, GTYPE2CLASS(GTK_TYPE_TEXT_MARK)))
         gtk_text_buffer_move_mark(_SELF(self), RVAL2MARK(mark), RVAL2ITR(where));
@@ -714,8 +637,7 @@ txt_move_mark(self, mark, where)
 }
 
 static VALUE
-txt_create_tag(self, tag_name, properties)
-    VALUE self, tag_name, properties;
+rg_create_tag(VALUE self, VALUE tag_name, VALUE properties)
 {
     GtkTextTag *tag;
     VALUE ret;
@@ -724,7 +646,7 @@ txt_create_tag(self, tag_name, properties)
     gtk_text_tag_table_add (gtk_text_buffer_get_tag_table(_SELF(self)), tag);
 
     G_SET_PROPERTIES(GOBJ2RVAL(tag), properties);
-    
+
     ret = GOBJ2RVAL(tag);
 
     G_CHILD_ADD(self, ret);
@@ -733,10 +655,7 @@ txt_create_tag(self, tag_name, properties)
 }
 
 static VALUE
-txt_insert(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+rg_insert(int argc, VALUE *argv, VALUE self)
 {
     VALUE where, value, tags;
     gint start_offset;
@@ -757,26 +676,28 @@ txt_insert(argc, argv, self)
         start_offset = gtk_text_iter_get_offset(RVAL2ITR(where));
         StringValue(value);
         gtk_text_buffer_insert(_SELF(self), RVAL2ITR(where),
-                               RVAL2CSTR(value), RSTRING_LEN(value));
-        
+                               RSTRING_PTR(value), RSTRING_LEN(value));
+
         if (RARRAY_LEN(tags) == 0)
             return self;
 
+        /* TODO: Do we really want to do this before we know that everything
+         * below passed without any errors? */
         G_CHILD_ADD(self, tags);
-        
+
         gtk_text_buffer_get_iter_at_offset(_SELF(self), &start, start_offset);
-        
+
         for(i = 0; i < RARRAY_LEN(tags); i++) {
             GtkTextTag *tag;
-            
+
             if (rb_obj_is_kind_of(RARRAY_PTR(tags)[i], GTYPE2CLASS(GTK_TYPE_TEXT_TAG))) {
                 tag = RVAL2GOBJ(RARRAY_PTR(tags)[i]);
             } else {
                 tag = gtk_text_tag_table_lookup(_SELF(self)->tag_table,
-						RVAL2CSTR(RARRAY_PTR(tags)[i]));
+                                                RVAL2CSTR(RARRAY_PTR(tags)[i]));
                 if (tag == NULL) {
                     g_warning ("%s: no tag with name '%s'!",
-			       G_STRLOC, RVAL2CSTR(RARRAY_PTR(tags)[i]));
+                               G_STRLOC, RVAL2CSTR(RARRAY_PTR(tags)[i]));
                     return self;
                 }
             }
@@ -787,20 +708,16 @@ txt_insert(argc, argv, self)
 }
 
 static VALUE
-txt_insert_with_tags(argc, argv, self)
-    int argc;
-    VALUE *argv;
-    VALUE self;
+rg_insert_with_tags(int argc, VALUE *argv, VALUE self)
 {
     if (RVAL2CBOOL(ruby_debug))
         rb_warning("Gtk::TextBuffer#insert_with_tags is deprecated. Use Gtk::TextBuffer#insert instead.");
-    txt_insert(argc, argv, self);
+    rg_insert(argc, argv, self);
     return self;
 }
 
 static VALUE
-txt_apply_tag(self, tag, start, end)
-    VALUE self, tag, start, end;
+rg_apply_tag(VALUE self, VALUE tag, VALUE start, VALUE end)
 {
     if (rb_obj_is_kind_of(tag, GTYPE2CLASS(GTK_TYPE_TEXT_TAG)))
         gtk_text_buffer_apply_tag(_SELF(self), RVAL2TAG(tag), RVAL2ITR(start), RVAL2ITR(end));
@@ -811,8 +728,7 @@ txt_apply_tag(self, tag, start, end)
 }
 
 static VALUE
-txt_remove_tag(self, tag, start, end)
-    VALUE self, tag, start, end;
+rg_remove_tag(VALUE self, VALUE tag, VALUE start, VALUE end)
 {
     if (rb_obj_is_kind_of(tag, GTYPE2CLASS(GTK_TYPE_TEXT_TAG)))
         gtk_text_buffer_remove_tag(_SELF(self), RVAL2TAG(tag), RVAL2ITR(start), RVAL2ITR(end));
@@ -823,16 +739,14 @@ txt_remove_tag(self, tag, start, end)
 }
 
 static VALUE
-txt_remove_all_tags(self, start, end)
-    VALUE self, start, end;
+rg_remove_all_tags(VALUE self, VALUE start, VALUE end)
 {
     gtk_text_buffer_remove_all_tags(_SELF(self), RVAL2ITR(start), RVAL2ITR(end));
     return self;
 }
 
 static VALUE
-txt_get_iter_at_line_offset(self, line_number, char_offset)
-    VALUE self, line_number, char_offset;
+rg_get_iter_at_line_offset(VALUE self, VALUE line_number, VALUE char_offset)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_iter_at_line_offset(_SELF(self), &iter, NUM2INT(line_number), NUM2INT(char_offset));
@@ -840,8 +754,7 @@ txt_get_iter_at_line_offset(self, line_number, char_offset)
 }
 
 static VALUE
-txt_get_iter_at_line_index(self, line_number, byte_index)
-    VALUE self, line_number, byte_index;
+rg_get_iter_at_line_index(VALUE self, VALUE line_number, VALUE byte_index)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_iter_at_line_index(_SELF(self), &iter, NUM2INT(line_number), NUM2INT(byte_index));
@@ -849,8 +762,7 @@ txt_get_iter_at_line_index(self, line_number, byte_index)
 }
 
 static VALUE
-txt_get_iter_at_offset(self, char_offset)
-    VALUE self, char_offset;
+rg_get_iter_at_offset(VALUE self, VALUE char_offset)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_iter_at_offset(_SELF(self), &iter, NUM2INT(char_offset));
@@ -858,8 +770,7 @@ txt_get_iter_at_offset(self, char_offset)
 }
 
 static VALUE
-txt_get_iter_at_line(self, line_number)
-    VALUE self, line_number;
+rg_get_iter_at_line(VALUE self, VALUE line_number)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_iter_at_line(_SELF(self), &iter, NUM2INT(line_number));
@@ -867,12 +778,11 @@ txt_get_iter_at_line(self, line_number)
 }
 
 static VALUE
-txt_get_bounds(self)
-    VALUE self;
+rg_bounds(VALUE self)
 {
     GtkTextIter start, end;
     VALUE result;
-    
+
     gtk_text_buffer_get_bounds(_SELF(self), &start, &end);
     result = rb_ary_new();
     rb_ary_push(result, ITR2RVAL(&start));
@@ -882,112 +792,107 @@ txt_get_bounds(self)
 }
 
 static VALUE
-txt_get_iter_at_mark(self, mark)
-    VALUE self, mark;
+rg_get_iter_at_mark(VALUE self, VALUE mark)
 {
     GtkTextIter iter;
     gtk_text_buffer_get_iter_at_mark(_SELF(self), &iter, RVAL2MARK(mark));
     return ITR2RVAL(&iter);
 }
 
-
 void 
-Init_gtk_textbuffer()
+Init_gtk_textbuffer(VALUE mGtk)
 {
-    VALUE gTextBuffer = G_DEF_CLASS(GTK_TYPE_TEXT_BUFFER, "TextBuffer", mGtk);
+    VALUE RG_TARGET_NAMESPACE = G_DEF_CLASS(GTK_TYPE_TEXT_BUFFER, "TextBuffer", mGtk);
 
     id_tagtable = rb_intern("tagtable");
 
-    rb_define_method(gTextBuffer, "initialize", txt_initialize, -1);
-    rb_define_method(gTextBuffer, "line_count", txt_get_line_count, 0);
-    rb_define_method(gTextBuffer, "char_count", txt_get_char_count, 0);
-    rb_define_method(gTextBuffer, "tag_table", txt_get_tag_table, 0);
+    RG_DEF_METHOD(initialize, -1);
+    RG_DEF_METHOD(line_count, 0);
+    RG_DEF_METHOD(char_count, 0);
 
-    rb_define_method(gTextBuffer, "set_text", txt_set_text, 1);
-    rb_define_method(gTextBuffer, "insert", txt_insert, -1);
-    rb_define_method(gTextBuffer, "insert_with_tags", txt_insert_with_tags, -1);
+    G_REPLACE_SET_PROPERTY(RG_TARGET_NAMESPACE, "text", txt_set_text, 1);
+    RG_DEF_METHOD(insert, -1);
+    RG_DEF_METHOD(insert_with_tags, -1);
 #if GTK_CHECK_VERSION(2,6,0)
-    rb_define_method(gTextBuffer, "backspace", txt_backspace, 3);
+    RG_DEF_METHOD(backspace, 3);
 #endif
-    rb_define_method(gTextBuffer, "insert_at_cursor", txt_insert_at_cursor, 1);
-    rb_define_method(gTextBuffer, "insert_interactive", txt_insert_interactive, 3);
-    rb_define_method(gTextBuffer, "insert_interactive_at_cursor", txt_insert_interactive_at_cursor, 2);
-    rb_define_method(gTextBuffer, "insert_range", txt_insert_range, 3);
-    rb_define_method(gTextBuffer, "insert_range_interactive", txt_insert_range_interactive, 4);
+    RG_DEF_METHOD(insert_at_cursor, 1);
+    RG_DEF_METHOD(insert_interactive, 3);
+    RG_DEF_METHOD(insert_interactive_at_cursor, 2);
+    RG_DEF_METHOD(insert_range, 3);
+    RG_DEF_METHOD(insert_range_interactive, 4);
 
-    rb_define_method(gTextBuffer, "delete", txt_delete, 2);
-    rb_define_method(gTextBuffer, "delete_interactive", txt_delete_interactive, 3);
+    RG_DEF_METHOD(delete, 2);
+    RG_DEF_METHOD(delete_interactive, 3);
 
-    rb_define_method(gTextBuffer, "get_text", txt_get_text, -1);
-    rb_define_method(gTextBuffer, "text", txt_get_text_all, 0);
-    rb_define_method(gTextBuffer, "get_slice", txt_get_slice, -1);
-    rb_define_method(gTextBuffer, "slice", txt_get_slice_all, 0);
+    RG_DEF_METHOD(get_text, -1);
+    G_REPLACE_GET_PROPERTY(RG_TARGET_NAMESPACE, "text", txt_get_text_all, 0);
+    RG_DEF_METHOD(get_slice, -1);
+    RG_DEF_METHOD(slice, 0);
 
-    rb_define_method(gTextBuffer, "insert_pixbuf", txt_insert_pixbuf, 2);
-    rb_define_method(gTextBuffer, "insert_child_anchor", txt_insert_child_anchor, 2);
-    rb_define_method(gTextBuffer, "create_child_anchor", txt_create_child_anchor, 1);
-    
-    rb_define_method(gTextBuffer, "create_mark", txt_create_mark, 3);
+    RG_DEF_METHOD(insert_pixbuf, 2);
+    RG_DEF_METHOD(insert_child_anchor, 2);
+    RG_DEF_METHOD(create_child_anchor, 1);
+
+    RG_DEF_METHOD(create_mark, 3);
 #if GTK_CHECK_VERSION(2,12,0)
-    rb_define_method(gTextBuffer, "add_mark", txt_add_mark, 2);
+    RG_DEF_METHOD(add_mark, 2);
 #endif
-    rb_define_method(gTextBuffer, "delete_mark", txt_delete_mark, 1);
+    RG_DEF_METHOD(delete_mark, 1);
 
-    rb_define_method(gTextBuffer, "get_mark", txt_get_mark, 1);
+    RG_DEF_METHOD(get_mark, 1);
 /* Comment out because this method's name is very bad.
    Use Gtk::TextBuffer#get_mark("insert") instead.
-    rb_define_method(gTextBuffer, "get_insert", txt_get_insert, 0);
+    RG_DEF_METHOD(get_insert, 0);
 */
-    rb_define_method(gTextBuffer, "selection_bound", txt_get_selection_bound, 0);
-    rb_define_method(gTextBuffer, "place_cursor", txt_place_cursor, 1);
+    RG_DEF_METHOD(selection_bound, 0);
+    RG_DEF_METHOD(place_cursor, 1);
 #if GTK_CHECK_VERSION(2,4,0)
-    rb_define_method(gTextBuffer, "select_range", txt_select_range, 2);
+    RG_DEF_METHOD(select_range, 2);
 #endif
-    rb_define_method(gTextBuffer, "modified?", txt_get_modified, 0);
-    rb_define_method(gTextBuffer, "set_modified", txt_set_modified, 1);
+    RG_DEF_METHOD_P(modified, 0);
+    RG_DEF_METHOD(set_modified, 1);
+    G_DEF_SETTER(RG_TARGET_NAMESPACE, "modified");
 
-    rb_define_method(gTextBuffer, "add_selection_clipboard", txt_add_selection_clipboard, 1);
-    rb_define_method(gTextBuffer, "remove_selection_clipboard", txt_remove_selection_clipboard, 1);
+    RG_DEF_METHOD(add_selection_clipboard, 1);
+    RG_DEF_METHOD(remove_selection_clipboard, 1);
 #if GTK_CHECK_VERSION(2,10,0)
-    rb_define_method(gTextBuffer, "deserialize", txt_deserialize, 4);
-    rb_define_method(gTextBuffer, "deserialize_can_create_tags?", txt_deserialize_get_can_create_tags, 1);
-    rb_define_method(gTextBuffer, "deserialize_set_can_create_tags", txt_deserialize_set_can_create_tags, 2);
-    rb_define_method(gTextBuffer, "deserialize_formats", txt_get_deserialize_formats, 0);
-    rb_define_method(gTextBuffer, "serialize_formats", txt_get_serialize_formats, 0);
-    rb_define_method(gTextBuffer, "register_deserialize_format", txt_register_deserialize_format, 1);
-    rb_define_method(gTextBuffer, "register_deserialize_target", txt_register_deserialize_target, 1);
-    rb_define_method(gTextBuffer, "register_serialize_format", txt_register_serialize_format, 1);
-    rb_define_method(gTextBuffer, "register_serialize_target", txt_register_serialize_target, 1);
-    rb_define_method(gTextBuffer, "serialize", txt_serialize, 4);
-    rb_define_method(gTextBuffer, "unregister_deserialize_format", txt_unregister_deserialize_format, 1);
-    rb_define_method(gTextBuffer, "unregister_serialize_format", txt_unregister_serialize_format, 1);
+    RG_DEF_METHOD(deserialize, 4);
+    RG_DEF_METHOD_P(deserialize_can_create_tags, 1);
+    RG_DEF_METHOD(deserialize_set_can_create_tags, 2);
+    RG_DEF_METHOD(deserialize_formats, 0);
+    RG_DEF_METHOD(serialize_formats, 0);
+    RG_DEF_METHOD(register_deserialize_format, 1);
+    RG_DEF_METHOD(register_deserialize_target, 1);
+    RG_DEF_METHOD(register_serialize_format, 1);
+    RG_DEF_METHOD(register_serialize_target, 1);
+    RG_DEF_METHOD(serialize, 4);
+    RG_DEF_METHOD(unregister_deserialize_format, 1);
+    RG_DEF_METHOD(unregister_serialize_format, 1);
 #endif
-    rb_define_method(gTextBuffer, "cut_clipboard", txt_cut_clipboard, 2);
-    rb_define_method(gTextBuffer, "copy_clipboard", txt_copy_clipboard, 1);
-    rb_define_method(gTextBuffer, "paste_clipboard", txt_paste_clipboard, 3);
-    
-    rb_define_method(gTextBuffer, "selection_bounds", txt_get_selection_bounds, 0);
-    rb_define_method(gTextBuffer, "delete_selection", txt_delete_selection, -1);
+    RG_DEF_METHOD(cut_clipboard, 2);
+    RG_DEF_METHOD(copy_clipboard, 1);
+    RG_DEF_METHOD(paste_clipboard, 3);
 
-    rb_define_method(gTextBuffer, "begin_user_action", txt_begin_user_action, 0);
-    rb_define_method(gTextBuffer, "end_user_action", txt_end_user_action, 0);
+    RG_DEF_METHOD(selection_bounds, 0);
+    RG_DEF_METHOD(delete_selection, -1);
 
-    rb_define_method(gTextBuffer, "start_iter", txt_get_start_iter, 0);
-    rb_define_method(gTextBuffer, "end_iter", txt_get_end_iter, 0);
-    rb_define_method(gTextBuffer, "get_iter_at_line_offset", txt_get_iter_at_line_offset, 2);
-    rb_define_method(gTextBuffer, "get_iter_at_line_index", txt_get_iter_at_line_index, 2);
-    rb_define_method(gTextBuffer, "get_iter_at_offset", txt_get_iter_at_offset, 1);
-    rb_define_method(gTextBuffer, "get_iter_at_line", txt_get_iter_at_line, 1);
-    rb_define_method(gTextBuffer, "bounds", txt_get_bounds, 0);
-    rb_define_method(gTextBuffer, "get_iter_at_mark", txt_get_iter_at_mark, 1);
-    rb_define_method(gTextBuffer, "move_mark", txt_move_mark, 2);
-    rb_define_method(gTextBuffer, "get_iter_at_child_anchor", txt_get_iter_at_child_anchor, 1);
-    
-    rb_define_method(gTextBuffer, "create_tag", txt_create_tag, 2);
-    rb_define_method(gTextBuffer, "apply_tag", txt_apply_tag, 3);
-    rb_define_method(gTextBuffer, "remove_tag", txt_remove_tag, 3);
-    rb_define_method(gTextBuffer, "remove_all_tags", txt_remove_all_tags, 2);
+    RG_DEF_METHOD(begin_user_action, 0);
+    RG_DEF_METHOD(end_user_action, 0);
 
-    G_DEF_SETTERS(gTextBuffer);
+    RG_DEF_METHOD(start_iter, 0);
+    RG_DEF_METHOD(end_iter, 0);
+    RG_DEF_METHOD(get_iter_at_line_offset, 2);
+    RG_DEF_METHOD(get_iter_at_line_index, 2);
+    RG_DEF_METHOD(get_iter_at_offset, 1);
+    RG_DEF_METHOD(get_iter_at_line, 1);
+    RG_DEF_METHOD(bounds, 0);
+    RG_DEF_METHOD(get_iter_at_mark, 1);
+    RG_DEF_METHOD(move_mark, 2);
+    RG_DEF_METHOD(get_iter_at_child_anchor, 1);
+
+    RG_DEF_METHOD(create_tag, 2);
+    RG_DEF_METHOD(apply_tag, 3);
+    RG_DEF_METHOD(remove_tag, 3);
+    RG_DEF_METHOD(remove_all_tags, 2);
 }
-
