@@ -22,11 +22,44 @@ $LOAD_PATH.unshift(mkmf_gnome2_dir.to_s)
 module_name = "pango"
 package_id = "pango"
 
-require 'mkmf-gnome2'
+begin
+  require 'mkmf-gnome2'
+rescue LoadError
+  require 'rubygems'
+  gem 'glib2'
+  require 'mkmf-gnome2'
+end
+
+ruby_header = "ruby.h"
+have_func("rb_errinfo", ruby_header)
+
+["glib2"].each do |package|
+  directory = "#{package}#{version_suffix}"
+  build_dir = "#{directory}/tmp/#{RUBY_PLATFORM}/#{package}/#{RUBY_VERSION}"
+  add_depend_package(package, "#{directory}/ext/#{package}",
+                     top_dir.to_s,
+                     :top_build_dir => top_build_dir.to_s,
+                     :target_build_dir => build_dir)
+end
+
+rcairo_options = {}
+rcairo_source_dir_names = ["rcairo"]
+if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
+  rcairo_source_dir_names.unshift("rcairo.win32")
+end
+rcairo_source_dir_names.each do |rcairo_source_dir_name|
+  rcairo_source_dir = top_dir.parent.expand_path + rcairo_source_dir_name
+  if rcairo_source_dir.exist?
+    rcairo_options[:rcairo_source_dir] = rcairo_source_dir.to_s
+    break
+  end
+end
+check_cairo(rcairo_options)
 
 setup_win32(module_name, base_dir)
 
 PKGConfig.have_package(package_id) or exit 1
+PKGConfig.have_package('pangocairo')
 
 pango_header = "pango/pango.h"
 have_func("pango_layout_iter_get_type", pango_header)
@@ -41,33 +74,6 @@ have_func("pango_attr_iterator_get_attrs", pango_header)
 have_func("pango_itemize_with_base_dir", pango_header)
 have_func("pango_font_family_is_monospace", pango_header)
 
-if PKGConfig.have_package('pangocairo')
-  options = {}
-  rcairo_source_dir_names = ["rcairo"]
-  if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
-    rcairo_source_dir_names.unshift("rcairo.win32")
-  end
-  rcairo_source_dir_names.each do |rcairo_source_dir_name|
-    rcairo_source_dir = top_dir.parent.expand_path + rcairo_source_dir_name
-    if rcairo_source_dir.exist?
-      options[:rcairo_source_dir] = rcairo_source_dir.to_s
-      break
-    end
-  end
-  check_cairo(options)
-end
-
-["glib2"].each do |package|
-  directory = "#{package}#{version_suffix}"
-  build_dir = "#{directory}/tmp/#{RUBY_PLATFORM}/#{package}/#{RUBY_VERSION}"
-  add_depend_package(package, "#{directory}/ext/#{package}",
-                     top_dir.to_s,
-                     :top_build_dir => top_build_dir.to_s,
-                     :target_build_dir => build_dir)
-end
-
-add_distcleanfile("rbpangoinits.c")
-
 make_version_header("PANGO", "pango", ".")
 
 create_pkg_config_file("Ruby/Pango", package_id)
