@@ -22,29 +22,15 @@ $LOAD_PATH.unshift(mkmf_gnome2_dir.to_s)
 module_name = "poppler"
 package_id = "poppler-glib"
 
-require 'mkmf-gnome2'
-
-setup_win32(module_name, base_dir)
-
-PKGConfig.have_package(package_id) or exit 1
-
-if PKGConfig.have_package('poppler-cairo')
-  options = {}
-  rcairo_source_dir_names = ["rcairo"]
-  if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
-    rcairo_source_dir_names.unshift("rcairo.win32")
-  end
-  rcairo_source_dir_names.each do |rcairo_source_dir_name|
-    rcairo_source_dir = top_dir.parent.expand_path + rcairo_source_dir_name
-    if rcairo_source_dir.exist?
-      options[:rcairo_source_dir] = rcairo_source_dir.to_s
-      break
-    end
-  end
-  check_cairo(options)
+begin
+  require 'mkmf-gnome2'
+rescue LoadError
+  require 'rubygems'
+  gem 'glib2'
+  require 'mkmf-gnome2'
 end
 
-["glib2", "gtk2", "gdk_pixbuf2"].each do |package|
+["glib2", "atk", "pango", "gdk_pixbuf2", "gtk2"].each do |package|
   directory = "#{package}#{version_suffix}"
   build_dir = "#{directory}/tmp/#{RUBY_PLATFORM}/#{package}/#{RUBY_VERSION}"
   add_depend_package(package, "#{directory}/ext/#{package}",
@@ -53,6 +39,24 @@ end
                      :target_build_dir => build_dir)
 end
 
+rcairo_options = {}
+rcairo_source_dir_names = ["rcairo"]
+if /mingw|cygwin|mswin32/ =~ RUBY_PLATFORM
+  rcairo_source_dir_names.unshift("rcairo.win32")
+end
+rcairo_source_dir_names.each do |rcairo_source_dir_name|
+  rcairo_source_dir = top_dir.parent.expand_path + rcairo_source_dir_name
+  if rcairo_source_dir.exist?
+    rcairo_options[:rcairo_source_dir] = rcairo_source_dir.to_s
+    break
+  end
+end
+check_cairo(rcairo_options)
+
+setup_win32(module_name, base_dir)
+
+PKGConfig.have_package(package_id) or exit 1
+
 unless have_macro("POPPLER_MAJOR_VERSION", ["poppler.h"])
   make_version_header("POPPLER", package_id, ".")
 end
